#!/usr/bin/env python3
# coding: utf-8

# Built-in packages

# External packages
import numpy as np

# Internal packages
from napoleontoolbox.utility.metrics import accuracy, sharpe, calmar

__all__ = ['set_text_stats']

# =========================================================================== #
#                              Printer Tools                                  #
# =========================================================================== #


def set_text_stats(underly, period=252, accur=True, perf=True, vol=True,
                   sharp=True, calma=True, underlying='Underlying', fees=0,
                   **kwpred):
    """ Set a table as string with different indicators (accuracy, perf, vol
    and sharpe) for underlying and several strategies.

    Parameters
    ----------
    underly : np.ndarray[ndim=1, dtype=np.float64]
        Series of underlying prices.
    period : int, optional
        Number of period per day, default is 252.
    accur : bool, optional
        If true compute accuracy else not, default is True.
    perf : bool, optional
        If true compute performance else not, default is True.
    vol : bool, optional
        If true compute volatility else not, default is True.
    sharp : bool, optional
        If true compute sharpe ratio else not, default is True.
    calma : bool, optional
        If true compute calmar ratio else not, default is True.
    underlying : str, optional
        Name of the underlying, default is 'Underlying'.
    kwpred : dict of np.ndarray
        Any strategies or predictions that you want to compare.
    fees : float, optional
        Fees to apply at the strategy performance.

    Returns
    -------
    txt : str
        Table of results.

    See Also
    --------
    PlotBackTest, display_perf

    """
    txt = ''
    # Compute Accuracy
    if accur:
        txt += '+=============================+\n'
        txt += '|          Accuracy           |\n'
        txt += '+----------------+------------+\n'
        for key, pred in kwpred.items():
            accu_pred = accuracy(underly, pred)
            txt += '| {:14} | {:10.2%} |\n'.format(key, accu_pred)
    # Compute performance
    if perf:
        txt += '+=============================+\n'
        txt += '|         Performance         |\n'
        txt += '+----------------+------------+\n'
        perf = np.exp(np.cumsum(underly))
        perf_targ = np.sign(perf[-1] / perf[0]) * np.float_power(
            np.abs(perf[-1] / perf[0]), period / perf.size) - 1.
        txt += '| {:14} | {:10.2%} |\n'.format(underlying, perf_targ)
        for key, pred in kwpred.items():
            vect_fee = np.zeros(pred.shape)
            vect_fee[1:] += np.abs(pred[1:] - pred[:-1]) * fees
            perf = np.exp(np.cumsum(underly * pred - vect_fee))
            perf_pred = np.sign(perf[-1] / perf[0]) * np.float_power(
                np.abs(perf[-1] / perf[0]), period / perf.size) - 1.
            txt += '| {:14} | {:10.2%} |\n'.format(key, perf_pred)
    # Compute volatility
    if vol:
        txt += '+=============================+\n'
        txt += '|          Volatility         |\n'
        txt += '+----------------+------------+\n'
        perf = np.exp(np.cumsum(underly))
        vol_targ = np.sqrt(period) * np.std(perf[1:] / perf[:-1] - 1)
        txt += '| {:14} | {:10.2%} |\n'.format(underlying, vol_targ)
        for key, pred in kwpred.items():
            vect_fee = np.zeros(pred.shape)
            vect_fee[1:] += np.abs(pred[1:] - pred[:-1]) * fees
            perf = np.exp(np.cumsum(underly * pred - vect_fee))
            vol_pred = np.sqrt(period) * np.std(perf[1:] / perf[:-1] - 1)
            txt += '| {:14} | {:10.2%} |\n'.format(key, vol_pred)
    # Compute sharpe Ratio
    if sharp:
        txt += '+=============================+\n'
        txt += '|         Sharpe Ratio        |\n'
        txt += '+----------------+------------+\n'
        sharpe_targ = sharpe(np.exp(np.cumsum(underly)), period=period)
        txt += '| {:14} | {:10.2f} |\n'.format(underlying, sharpe_targ)
        for key, pred in kwpred.items():
            vect_fee = np.zeros(pred.shape)
            vect_fee[1:] += np.abs(pred[1:] - pred[:-1]) * fees
            perf = np.exp(np.cumsum(underly * pred - vect_fee))
            sharpe_pred = sharpe(perf, period=period)
            txt += '| {:14} | {:10.2f} |\n'.format(key, sharpe_pred)
    # Compute calmar
    if calma:
        txt += '+=============================+\n'
        txt += '|         Calmar Ratio        |\n'
        txt += '+----------------+------------+\n'
        calmar_targ = calmar(np.exp(np.cumsum(underly)), period=period)
        txt += '| {:14} | {:10.2f} |\n'.format(underlying, calmar_targ)
        for key, pred in kwpred.items():
            vect_fee = np.zeros(pred.shape)
            vect_fee[1:] += np.abs(pred[1:] - pred[:-1]) * fees
            perf = np.exp(np.cumsum(underly * pred - vect_fee))
            calmar_pred = calmar(perf, period=period)
            txt += '| {:14} | {:10.2f} |\n'.format(key, calmar_pred)
    txt += '+=============================+\n'
    return txt



import json

from napoleontoolbox.napoleon_config_tools.object.modelparamtemplate import ModelParamTemplate
from npxlogger import log


# todo make meta dict




class _Config(object):
    FILE = "config_test"

    def __init__(self):
        self.__profile = None
        self.__package = None
        self.__isrunnable = bool()
        self.__package = str()
        self.__model_parameters = ModelParamTemplate()
        self.__providers = None

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        return json.dumps(self.__dict__())

    def __dict__(self):
        return dict(profile=self.__profile,
                    isrunnable=self.__isrunnable,
                    package=self.__package,
                    model_parameters=self.__model_parameters.__dict__())

    @classmethod
    def value(cls,_value: str):
        def inner_function(function):
            def wrapper(*args, **kwargs):
                log.info("target value in config :  {value}".format(value=_value))
                params = cls.__look_up(_value)
                function(*args, **kwargs)

            return wrapper

        return inner_function

    @staticmethod
    def __look_up(_value: str):
        _path = _value.split(".")
        value = ""# todo : gettattr
        return value

    @property
    def profile(self) -> str:
        return self.__profile

    @profile.setter
    def profile(self, profile: str):
        self.__profile = profile

    @property
    def providers (self) -> dict:
        return self.__providers

    @providers.setter
    def providers(self, _providers: dict):
        self.__providers = _providers

    @property
    def isrunnable(self) -> bool:
        return self.__isrunnable

    @isrunnable.setter
    def isrunnable(self, x: bool):
        self.__isrunnable = x

    @property
    def package(self):
        return self.__package

    @package.setter
    def package(self, _package_name: dict):
        self.__package = _package_name

    @property
    def model_parameters(self) -> ModelParamTemplate:
        return self.__model_parameters








Config = _Config



__author__ = "hugo.inzirillo"

from abc import abstractmethod, ABCMeta

from napoleontoolbox.authentication import Token


class AbstractAuthResponseHandlerTemplate(metaclass=ABCMeta):

    @property
    @abstractmethod
    def _response(self):
        raise NotImplemented

    @abstractmethod
    def _get_token(self):
        raise NotImplemented

    @abstractmethod
    def _get_token_type(self):
        raise NotImplemented

    @abstractmethod
    def get_token(self):
        raise NotImplemented


class AuthResponseHandlerTemplate(AbstractAuthResponseHandlerTemplate):
    def __init__(self, _reponse: dict):
        self.__reponse = _reponse

    @property
    def _response(self):
        return self.__reponse

    def _get_token(self): ...

    def _get_token_type(self): ...

    def get_token(self): ...


class NapoleonServiceAuthHandler(AuthResponseHandlerTemplate):

    def _get_token(self):
        return self._response["access_token"]

    def _get_token_type(self):
        return str(self._response["token_type"]).capitalize()

    def get_token(self):
        _token = Token()
        _token.value = self._get_token()
        _token.type = self._get_token_type()
        return _token



__author__ = "hugo.inzirillo"

from abc import abstractmethod, ABCMeta

import requests
import requests.auth

from napoleontoolbox.authentication import Token, ClientCredentials, PasswordCredentials, Scope
from napoleontoolbox.authentication.auth_handler import NapoleonServiceAuthHandler
from napoleontoolbox.authentication.security_headers import AuthorizationHeader, ApplicationJsonHeader
from napoleontoolbox.dataloader.models import NapoleonService
from napoleontoolbox.napoleon_config_tools import getter


class AuthorizationHeaderManager(object):
    def __init__(self, token: Token):
        self.__token = token

    def build(self):
        return AuthorizationHeader.build(self.__token)


class AbstractAuthManagerTemplate(metaclass=ABCMeta):

    @abstractmethod
    def authenticate(self) -> Token: ...

    @abstractmethod
    def _request(self) -> requests.Request: ...

    @abstractmethod
    def _handle_reponse(self) -> requests.Request: ...

    @abstractmethod
    def _data(self): ...

    @abstractmethod
    def _auth(self): ...

    @property
    @abstractmethod
    def response(self): ...

    @property
    @abstractmethod
    def token(self): ...

    @property
    @abstractmethod
    def authorization_header(self): ...


class AuthManagerTemplate(AbstractAuthManagerTemplate):
    def __init__(self):
        self.__url = str()
        self.__credentials = None
        self.__scope = Scope()
        self.__response = requests.Response
        self.__token = Token()
        self.__authorization_header = str()

    @property
    def url(self):
        return self.__url

    @property
    def credentials(self):
        return self.__credentials

    @property
    def scope(self):
        return self.__scope

    @property
    def response(self):
        return self.__response

    @response.setter
    def response(self, response):
        self.__response = response

    @property
    def authorization_header(self):
        return self.__authorization_header

    @authorization_header.setter
    def authorization_header(self, header: str):
        self.__authorization_header = header

    @property
    def token(self):
        return self.__token

    def _request(self) -> requests.Request: ...

    def _data(self): ...

    def _auth(self): ...

    def authenticate(self): ...


class AuthenticationManager(AuthManagerTemplate):

    def _auth(self):
        return requests.auth.HTTPBasicAuth(self.credentials.client_id, self.credentials.client_secret)

    @property
    def token(self):
        return self.__token

    @token.setter
    def token(self, token):
        self.__token = token

    def _data(self):
        _temp = None
        if isinstance(self.credentials, ClientCredentials):
            _temp = {"grant_type": "{grant_type}"}
            _temp.update({"grant_type": self.credentials.__repr__()})
        elif isinstance(self.credentials, PasswordCredentials):
            _temp = {"username": "{username}", "password": "{password}", "grant_type": "{grant_type}"}
            _temp.update(
                {"username": self.credentials.user, "password": self.credentials.password,
                 "grant_type": self.credentials.__repr__()}
            )
        if _temp:
            return _temp
        return {}

    def _request_token(self):
        if self.url and self.credentials and self.scope:
            self.response = self._request()
            self._handle_reponse()

    def _handle_reponse(self):
        raise NotImplemented

    def _request(self):
        return requests.post(url=self.url, headers=ApplicationJsonHeader().build(), auth=self._auth(),
                             data=self._data())

    def authenticate(self):
        self._request_token()


class NapoleonAuthenticationManager(AuthenticationManager):

    @property
    @getter(value="providers.napoleon_service.access_token_uri")
    def url(self):
        return self.url

    @property
    def credentials(self):
        return NapoleonService().get_credentials()

    @property
    def scope(self):
        return Scope(["READ", "WRITE"])

    def _handle_reponse(self):
        if self.response:
            if self.response.status_code == requests.codes.OK:
                _reponse = self.response.json()
                self.token = NapoleonServiceAuthHandler(_reponse).get_token()
                self.authorization_header = AuthorizationHeaderManager(self.token).build()


class NapoleonConnectorManager(object):

    def __init__(self):
        self.__authentication_manager = NapoleonAuthenticationManager()
        
    @property
    def authentication_manager(self):
        return self.__authentication_manager

    def execute_request(self):
        self.authentication_manager.authenticate()


if __name__ == '__main__':
    npx_auth_manager = NapoleonAuthenticationManager()
    npx_auth_manager.authenticate()

    end = True



__author__ = "hugo.inzirillo"

from abc import abstractmethod, ABCMeta
from enum import Enum


class AbstractCredentials(metaclass=ABCMeta):

    @abstractmethod
    def _client_id(self):
        raise NotImplemented

    @abstractmethod
    def _client_secret(self):
        raise NotImplemented

    @abstractmethod
    def _user(self):
        raise NotImplemented

    @abstractmethod
    def _password(self):
        raise NotImplemented


class ClientCredentials(AbstractCredentials):
    def __init__(self, client_id: str = None, client_secret: str = None):
        self.__client_id = client_id
        self.__client_secret = client_secret

    @property
    def client_id(self):
        return self._client_id

    @property
    def _client_id(self):
        return self.__client_id

    @property
    def client_secret(self):
        return self.__client_secret

    @property
    def _client_secret(self):
        return self.__client_secret

    @property
    def _user(self):
        return None

    @property
    def _password(self):
        return None

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        return "client_credentials"


class PasswordCredentials(AbstractCredentials):
    def __init__(self, client_id: str = None, client_secret: str = None,
                 user: str = None, password: str = None):
        self.__client_id = client_id
        self.__client_secret = client_secret
        self.__user = user
        self.__password = password

    @property
    def client_id(self):
        return self._client_id

    @property
    def _client_id(self):
        return self.__client_id

    @property
    def client_secret(self):
        return self.__client_secret

    @property
    def _client_secret(self):
        return self.__client_secret

    @property
    def user(self):
        return self._user

    @property
    def _user(self):
        return self.__user

    @property
    def password(self):
        return self._password

    @property
    def _password(self):
        return self.__password

    def __repr__(self):
        return self.__repr__()

    def __str__(self):
        return "password_credentials"


class TokenType(Enum):
    BEARER = "Bearer"
    BASIC = "Basic"


class Token(object):
    def __init__(self):
        self.__value = None
        self.__type = TokenType.BEARER.value

    def __repr__(self):
        return self.__type + " " + self.__value

    @property
    def value(self):
        return self.__value

    @property
    def type(self):
        return self.__type

    @value.setter
    def value(self, value):
        self.__value = value

    @type.setter
    def type(self, type):
        self.__type = type


class Scope(list):
    pass



#!/usr/bin/env python
# coding: utf-8

from napoleontoolbox.cointegration_utils import cointegration_utilities
import numpy as np

# MAX_PERIOD_TO_KEEP_POSITION = 0
# PREVIOUS_SIGNAL_TYPE = None


def pairs_trading_simple_correlation(pair_df = None, correlation_threshold = 0.2, suffixes=('_btc', '_eth'), **kwargs):
    log_returns_df = cointegration_utilities.add_log_returns(pair_df, suffixes=suffixes)
    y,x = log_returns_df['log_return'+suffixes[1]],log_returns_df['log_return'+suffixes[0]]

    y_cum_ret = log_returns_df['log_return' + suffixes[1]].sum()
    x_cum_ret = log_returns_df['log_return' + suffixes[0]].sum()

    correlation_matrix = np.corrcoef(x, y)
    corr_coef = correlation_matrix[0][1]
    trend = np.sign(corr_coef)
    magnitude = abs(corr_coef)

    if magnitude >= correlation_threshold:
        if trend == 1:
            if x_cum_ret>y_cum_ret:
                return -1,1
            else :
                return 1,-1
        else:
            if x_cum_ret>y_cum_ret:
                return 1,-1
            else :
                return -1,1
    else:
        np.nan,np.nan

    return None


def pair_trading_shifted_correlation(pair_df = None, correlation_range = 100, variant = 'a', suffixes=('_btc', '_eth'), **kwargs):
    len_histo = len(pair_df)
    assert len_histo > correlation_range
    return_pair_df = cointegration_utilities.add_returns(pair_df, suffixes=suffixes)
    correlation_dico = {}
    for shift in range(correlation_range):
        shifted_df = return_pair_df.shift(shift).copy()
        shifted_df['return' + suffixes[1]] = return_pair_df['return'+suffixes[1]].shift(shift)
        correlation_dico[shift] = shifted_df[['return'+suffixes[0], 'return'+suffixes[1]]].corr().values[0][1]
    lag = max(correlation_dico, key=correlation_dico.get)
    if variant == 'a':
        signal = return_pair_df['return' + suffixes[1]].values[-lag]
        return -np.sign(signal), np.sign(signal)

    elif variant == 'b':
        signal = np.mean(return_pair_df['return' + suffixes[1]].values[-lag:])
        return np.sign(signal), -np.sign(signal)

def pair_trading_shifted_correlation_with_cointegration(pair_df = None, correlation_range = 100, risk = 10, alpha = 0.05, variant = 'a', suffixes=('_btc', '_eth'), **kwargs):
    len_histo = len(pair_df)
    assert len_histo > correlation_range
    log_returns_df = cointegration_utilities.add_log_returns(pair_df, suffixes=suffixes)
    log_returns_df = cointegration_utilities.compute_log_return_regressor(log_returns_df, suffixes=suffixes)
    return_pair_df = cointegration_utilities.add_returns(log_returns_df, suffixes=suffixes)
    coint_test, alpha, z = cointegration_utilities.apply_cointegration_test(return_pair_df, risk=risk, alpha=alpha,
                                                                            suffixes=suffixes)
    if not coint_test:
        correlation_dico = {}
        for shift in range(correlation_range):
            shifted_df = return_pair_df.shift(shift).copy()
            shifted_df['return' + suffixes[1]] = return_pair_df['return'+suffixes[1]].shift(shift)
            correlation_dico[shift] = shifted_df[['return'+suffixes[0], 'return'+suffixes[1]]].corr().values[0][1]
        lag = max(correlation_dico, key=correlation_dico.get)

        if variant == 'a':
            signal = return_pair_df['return'+suffixes[1]].values[-lag]
            return -np.sign(signal), np.sign(signal)

        elif variant == 'b':
            signal = np.mean(return_pair_df['return'+suffixes[1]].values[-lag:])
            return np.sign(signal), -np.sign(signal)

    else:
        return np.nan,np.nan


def pairs_trading_log_simple_baseline_a(pair_df=None, percent_deviation = 0.1, risk = 10, alpha = 0.05, suffixes=('_btc', '_eth'), log_regressor_kind='log_x_over_log_y', **kwargs):
    log_returns_df = cointegration_utilities.add_log_returns(pair_df, suffixes=suffixes)
    log_returns_df = cointegration_utilities.compute_log_return_regressor(log_returns_df, suffixes=suffixes)
    coint_test, alpha, z = cointegration_utilities.apply_cointegration_test(log_returns_df, risk=risk, alpha = alpha, suffixes=suffixes)
    data_df = log_returns_df.copy()

    udl_x = 'flat'
    udl_y = 'flat'
    current_signal_type = None
    # global MAX_PERIOD_TO_KEEP_POSITION, PREVIOUS_SIGNAL_TYPE

    if not coint_test: #If not cointegrated

        new_return_regressor = data_df[log_regressor_kind][-1]
        # last_mean_regressor = data_df[log_regressor_kind].mean()
        last_mean_regressor = data_df[log_regressor_kind][:-1].mean()

        actions_udl_x = {'long': 1, 'short': -1, 'flat': 0}
        actions_udl_y = {'long': 1, 'short': -1, 'flat': 0}

        if new_return_regressor>(1+percent_deviation)*last_mean_regressor:
            current_signal_type = 'new_return_regressor>threshold'
            udl_x = 'short'
            udl_y = 'long'

        elif new_return_regressor<(1-percent_deviation)*last_mean_regressor:
            current_signal_type = 'new_return_regressor<threshold'
            udl_x = 'long'
            udl_y = 'short'

        return actions_udl_x[udl_x], actions_udl_y[udl_y]
        # if PREVIOUS_SIGNAL_TYPE != current_signal_type:
        #     PREVIOUS_SIGNAL_TYPE = current_signal_type
        #     MAX_PERIOD_TO_KEEP_POSITION = 1
        #     return actions_udl_x[udl_x], actions_udl_y[udl_y]
        #
        # else:
        #     if MAX_PERIOD_TO_KEEP_POSITION > keep_same_position_periods:
        #         udl_x = 'flat'
        #         udl_y = 'flat'
        #         return actions_udl_x[udl_x], actions_udl_y[udl_y]
        #     else:
        #         MAX_PERIOD_TO_KEEP_POSITION += 1
        #         return actions_udl_x[udl_x], actions_udl_y[udl_y]
    else:
        return 0, 0


def pairs_trading_log_simple_baseline_b(pair_df=None, percent_deviation = 0.1, risk = 10, alpha = 0.05, suffixes=('_btc', '_eth'), log_regressor_kind='log_x_over_log_y', **kwargs):
    log_returns_df = cointegration_utilities.add_log_returns(pair_df, suffixes=suffixes)
    log_returns_df = cointegration_utilities.compute_log_return_regressor(log_returns_df, suffixes=suffixes)
    coint_test, alpha, z = cointegration_utilities.apply_cointegration_test(log_returns_df, risk=risk, alpha = alpha, suffixes=suffixes)
    data_df = log_returns_df.copy()

    udl_x = 'flat'
    udl_y = 'flat'
    current_signal_type = None
    # global MAX_PERIOD_TO_KEEP_POSITION, PREVIOUS_SIGNAL_TYPE

    if not coint_test: #If not cointegrated

        new_return_regressor = data_df[log_regressor_kind][-1]
        # last_mean_regressor = data_df[log_regressor_kind].mean()
        last_mean_regressor = data_df[log_regressor_kind][:-1].mean()

        actions_udl_x = {'long': 1, 'short': -1, 'flat': 0}
        actions_udl_y = {'long': abs(new_return_regressor), 'short': -abs(new_return_regressor), 'flat': 0}

        if new_return_regressor>(1+percent_deviation)*last_mean_regressor:
            current_signal_type = 'new_return_regressor>threshold'
            udl_x = 'short'
            udl_y = 'long'

        elif new_return_regressor<(1-percent_deviation)*last_mean_regressor:
            current_signal_type = 'new_return_regressor<threshold'
            udl_x = 'long'
            udl_y = 'short'

        return actions_udl_x[udl_x], actions_udl_y[udl_y]
        # if PREVIOUS_SIGNAL_TYPE != current_signal_type:
        #     PREVIOUS_SIGNAL_TYPE = current_signal_type
        #     MAX_PERIOD_TO_KEEP_POSITION = 1
        #     return actions_udl_x[udl_x], actions_udl_y[udl_y]
        #
        # else:
        #     if MAX_PERIOD_TO_KEEP_POSITION > keep_same_position_periods:
        #         udl_x = 'flat'
        #         udl_y = 'flat'
        #         return actions_udl_x[udl_x], actions_udl_y[udl_y]
        #     else:
        #         MAX_PERIOD_TO_KEEP_POSITION += 1
        #         return actions_udl_x[udl_x], actions_udl_y[udl_y]
    else:
        return 0, 0

def pairs_trading_log_zscore_baseline_a(pair_df=None, risk = 10, alpha = 0.05, skipping_point=5, threshold_type = 'mean', suffixes=('_btc', '_eth'), number_of_zscore_dev = 1, log_regressor_kind='log_x_over_log_y', **kwargs):
    log_returns_df = cointegration_utilities.add_log_returns(pair_df, suffixes=suffixes)
    log_returns_df = cointegration_utilities.compute_log_return_regressor(log_returns_df, suffixes=suffixes)
    log_mean_zscore_df = cointegration_utilities.log_return_regressor_half_mean_and_z_score(log_returns_df, log_regressor_kind=log_regressor_kind, skipping_point=skipping_point)
    data_df = log_mean_zscore_df.copy()
    coint_test, alpha, z = cointegration_utilities.apply_cointegration_test(data_df, risk=risk, alpha=alpha,
                                                                  suffixes=suffixes)
    udl_x = 'flat'
    udl_y = 'flat'
    current_signal_type = None
    # global MAX_PERIOD_TO_KEEP_POSITION, PREVIOUS_SIGNAL_TYPE

    if not coint_test: #If not cointegrated

        # last_half_mean = data_df.log_return_regressor_half_mean[-1]
        last_half_mean = data_df.log_return_regressor_half_mean[:-1][-1]
        new_zscore = data_df.log_return_regressor_zscore[-1]
        zscore_threshold = cointegration_utilities.compute_zscore_threshold(data_df, threshold_type)

        actions_udl_x = {'long': 1, 'short': -1, 'flat': 0}
        actions_udl_y = {'long': 1, 'short': -1, 'flat': 0}

        if zscore_threshold >= last_half_mean:
            if new_zscore >= number_of_zscore_dev*zscore_threshold:
                current_signal_type = 'new_zscore>zscore_threshold'
                udl_x = 'short'
                udl_y = 'long'

        else:
            if new_zscore <= number_of_zscore_dev*zscore_threshold:
                current_signal_type = 'new_zscore<zscore_threshold'
                udl_x = 'long'
                udl_y = 'short'

        return actions_udl_x[udl_x], actions_udl_y[udl_y]
        # if PREVIOUS_SIGNAL_TYPE != current_signal_type:
        #     PREVIOUS_SIGNAL_TYPE = current_signal_type
        #     MAX_PERIOD_TO_KEEP_POSITION = 1
        #     return actions_udl_x[udl_x], actions_udl_y[udl_y]
        #
        # else:
        #     if MAX_PERIOD_TO_KEEP_POSITION > keep_same_position_periods:
        #         udl_x = 'flat'
        #         udl_y = 'flat'
        #         return actions_udl_x[udl_x], actions_udl_y[udl_y]
        #     else:
        #         MAX_PERIOD_TO_KEEP_POSITION += 1
        #         return actions_udl_x[udl_x], actions_udl_y[udl_y]
    else:
        return 0, 0


def pairs_trading_log_zscore_baseline_b(pair_df=None, risk=10, alpha=0.05,
                                        skipping_point=5, threshold_type='mean', suffixes=('_btc', '_eth'),
                                        number_of_zscore_dev = 1, log_regressor_kind='log_x_over_log_y', **kwargs):
    log_returns_df = cointegration_utilities.add_log_returns(pair_df, suffixes=suffixes)
    log_returns_df = cointegration_utilities.compute_log_return_regressor(log_returns_df, suffixes=suffixes)
    log_mean_zscore_df = cointegration_utilities.log_return_regressor_half_mean_and_z_score(log_returns_df,
                                                                                            log_regressor_kind=log_regressor_kind,
                                                                                            skipping_point=skipping_point)
    data_df = log_mean_zscore_df.copy()
    coint_test, alpha, z = cointegration_utilities.apply_cointegration_test(data_df, risk=risk, alpha=alpha,
                                                                  suffixes=suffixes)
    udl_x = 'flat'
    udl_y = 'flat'
    current_signal_type = None
    # global MAX_PERIOD_TO_KEEP_POSITION, PREVIOUS_SIGNAL_TYPE

    if not coint_test:  # If not cointegrated

        # last_half_mean = data_df.log_return_regressor_half_mean[-1]
        last_half_mean = data_df.log_return_regressor_half_mean[:-1][-1]
        new_zscore = data_df.log_return_regressor_zscore[-1]
        zscore_threshold = cointegration_utilities.compute_zscore_threshold(data_df, threshold_type)
        new_return_regressor = data_df[log_regressor_kind][-1]

        actions_udl_x = {'long': 1, 'short': -1, 'flat': 0}
        actions_udl_y = {'long': abs(new_return_regressor), 'short': -abs(new_return_regressor), 'flat': 0}

        if zscore_threshold >= last_half_mean:
            if new_zscore >= number_of_zscore_dev*zscore_threshold:
                current_signal_type = 'new_zscore>zscore_threshold'
                udl_x = 'short'
                udl_y = 'long'

        else:
            if new_zscore <= number_of_zscore_dev*zscore_threshold:
                current_signal_type = 'new_zscore<zscore_threshold'
                udl_x = 'long'
                udl_y = 'short'

        return actions_udl_x[udl_x], actions_udl_y[udl_y]
        # if PREVIOUS_SIGNAL_TYPE != current_signal_type:
        #     PREVIOUS_SIGNAL_TYPE = current_signal_type
        #     MAX_PERIOD_TO_KEEP_POSITION = 1
        #     return actions_udl_x[udl_x], actions_udl_y[udl_y]
        #
        # else:
        #     if MAX_PERIOD_TO_KEEP_POSITION > keep_same_position_periods:
        #         udl_x = 'flat'
        #         udl_y = 'flat'
        #         return actions_udl_x[udl_x], actions_udl_y[udl_y]
        #     else:
        #         MAX_PERIOD_TO_KEEP_POSITION += 1
        #         return actions_udl_x[udl_x], actions_udl_y[udl_y]
    else:
        return 0, 0

def pairs_trading_log_ecm_baseline_a(pair_df=None, risk = 10, alpha = 0.05, mean_lag = 5, ecm_shift = 0.05, suffixes=('_btc', '_eth'), **kwargs):
    """ Using Error Correction Model """
    log_returns_df = cointegration_utilities.add_log_returns(pair_df, suffixes=suffixes)
    log_returns_df = cointegration_utilities.compute_log_return_regressor(log_returns_df, suffixes=suffixes)
    data_df = log_returns_df.copy()
    coint_test, alpha, z = cointegration_utilities.apply_cointegration_test(data_df, risk=risk, alpha=alpha,
                                                                            suffixes=suffixes)
    data_df = cointegration_utilities.add_ecm_result(data_df, alpha, z)

    udl_x = 'flat'
    udl_y = 'flat'
    current_signal_type = None
    # global MAX_PERIOD_TO_KEEP_POSITION, PREVIOUS_SIGNAL_TYPE

    if not coint_test:  # If not cointegrated
        # last_moving_average_ecm = data_df.ecm.rolling(mean_lag).mean()[-1]
        last_moving_average_ecm = data_df.ecm.rolling(mean_lag).mean()[:-1][-1]
        new_ecm = data_df.ecm[-1]

        actions_udl_x = {'long': 1, 'short': -1, 'flat': 0}
        actions_udl_y = {'long': 1, 'short': -1, 'flat': 0}

        if new_ecm >= 0:
            if new_ecm >= last_moving_average_ecm + ecm_shift:
                current_signal_type = 'new_ecm >= last_moving_average_ecm + ecm_shift'
                udl_x = 'short'
                udl_y = 'long'

        else:
            if new_ecm <= last_moving_average_ecm - ecm_shift:
                current_signal_type = 'new_ecm <= last_moving_average_ecm - ecm_shift'
                udl_x = 'long'
                udl_y = 'short'

        return actions_udl_x[udl_x], actions_udl_y[udl_y]
        # if PREVIOUS_SIGNAL_TYPE != current_signal_type:
        #     PREVIOUS_SIGNAL_TYPE = current_signal_type
        #     MAX_PERIOD_TO_KEEP_POSITION = 1
        #     return actions_udl_x[udl_x], actions_udl_y[udl_y]
        #
        # else:
        #     if MAX_PERIOD_TO_KEEP_POSITION > keep_same_position_periods:
        #         udl_x = 'flat'
        #         udl_y = 'flat'
        #         return actions_udl_x[udl_x], actions_udl_y[udl_y]
        #     else:
        #         MAX_PERIOD_TO_KEEP_POSITION += 1
        #         return actions_udl_x[udl_x], actions_udl_y[udl_y]
    else:
        return 0, 0


def pairs_trading_log_ecm_baseline_b(pair_df=None, risk=10, alpha=0.05, mean_lag=5, ecm_shift=0.05, suffixes=('_btc', '_eth'), log_regressor_kind='log_x_over_log_y', **kwargs):
    """ Using Error Correction Model """
    log_returns_df = cointegration_utilities.add_log_returns(pair_df, suffixes=suffixes)
    log_returns_df = cointegration_utilities.compute_log_return_regressor(log_returns_df, suffixes=suffixes)
    data_df = log_returns_df.copy()
    coint_test, alpha, z = cointegration_utilities.apply_cointegration_test(data_df, risk=risk, alpha=alpha,
                                                                            suffixes=suffixes)
    data_df = cointegration_utilities.add_ecm_result(data_df, alpha, z)

    udl_x = 'flat'
    udl_y = 'flat'
    current_signal_type = None
    # global MAX_PERIOD_TO_KEEP_POSITION, PREVIOUS_SIGNAL_TYPE

    if not coint_test:  # If not cointegrated
        # last_moving_average_ecm = data_df.ecm.rolling(mean_lag).mean()[-1]
        last_moving_average_ecm = data_df.ecm.rolling(mean_lag).mean()[:-1][-1]
        new_ecm = data_df.ecm[-1]
        new_return_regressor = data_df[log_regressor_kind][-1]

        actions_udl_x = {'long': 1, 'short': -1, 'flat': 0}
        actions_udl_y = {'long': abs(new_return_regressor), 'short': -abs(new_return_regressor), 'flat': 0}

        if new_ecm >= 0:
            if new_ecm >= last_moving_average_ecm + ecm_shift:
                current_signal_type = 'new_ecm >= last_moving_average_ecm + ecm_shift'
                udl_x = 'short'
                udl_y = 'long'

        else:
            if new_ecm <= last_moving_average_ecm - ecm_shift:
                current_signal_type = 'new_ecm <= last_moving_average_ecm - ecm_shift'
                udl_x = 'long'
                udl_y = 'short'

        return actions_udl_x[udl_x], actions_udl_y[udl_y]
        # if PREVIOUS_SIGNAL_TYPE != current_signal_type:
        #     PREVIOUS_SIGNAL_TYPE = current_signal_type
        #     MAX_PERIOD_TO_KEEP_POSITION = 1
        #     return actions_udl_x[udl_x], actions_udl_y[udl_y]
        #
        # else:
        #     if MAX_PERIOD_TO_KEEP_POSITION > keep_same_position_periods:
        #         udl_x = 'flat'
        #         udl_y = 'flat'
        #         return actions_udl_x[udl_x], actions_udl_y[udl_y]
        #     else:
        #         MAX_PERIOD_TO_KEEP_POSITION += 1
        #         return actions_udl_x[udl_x], actions_udl_y[udl_y]
    else:
        return 0, 0



from plotly.subplots import make_subplots

import plotly.graph_objs as go

def generate_rebalanced_indice_weight_composition(weights_df = None):
    if weights_df is None:
        fig = make_subplots(rows=2, cols=1)
        fig.update_layout(height=600, width=1600, title_text="computations cannot be completed")
        return fig
    else:

        fig = make_subplots(rows=2, cols=1)

        fig.append_trace(
            go.Scatter(
                x=weights_df.index,
                y=weights_df.indice,
                name='Rebalanced Indice',
                opacity=0.8),
            row=1, col=1)

        constituents = [col for col in weights_df.columns if col not in ['indice']]
        for me_constituent in constituents:
            trace_sig = go.Scatter(
                x=weights_df.index,
                y=weights_df[me_constituent],
                name=me_constituent,
                opacity=0.8)
            fig.append_trace(trace_sig, row=2, col=1)


        fig.update_layout(height=600, width=1600, title_text="Weights composition",showlegend=True)
        #fig.update_xaxes(rangeslider_visible=True)
        return fig

def generate_rebalanced_indice(weights_df = None, title = 'Rebalanced Indice Value',  error_message = None):
    if weights_df is None:
        fig = make_subplots(rows=2, cols=1)
        if error_message is not None:
            fig.update_layout(height=600, width=1600, title_text=error_message)
        else:
            fig.update_layout(height=600, width=1600, title_text="computations cannot be completed")
        return fig
    else:
        fig = make_subplots(rows=1, cols=1)

        fig.append_trace(
            go.Scatter(
                x=weights_df.index,
                y=weights_df.indice,
                name='Rebalanced Indice',
                opacity=0.8),
            row=1, col=1)

        fig.update_layout(height=600, width=1600, title_text= title,showlegend=True)
        fig.update_xaxes(rangeslider_visible=True)
        return fig

def generate_weight_composition(weights_df = None, error_message = None):
    if weights_df is None:
        fig = make_subplots(rows=1, cols=1)
        if error_message is not None:
            fig.update_layout(height=600, width=1600, title_text=error_message)
        else:
            fig.update_layout(height=600, width=1600, title_text="computations cannot be completed")
        return fig
    else:
        if 'is_rebalancing' in list(weights_df.columns):
            weights_df = weights_df.drop(columns=['is_rebalancing'])
        fig = make_subplots(rows=1, cols=1)
        constituents = [col for col in weights_df.columns if col not in ['indice']]
        for me_constituent in constituents:
            trace_sig = go.Scatter(
                x=weights_df.index,
                y=weights_df[me_constituent],
                name=me_constituent,
                opacity=0.8)
            fig.append_trace(trace_sig, row=1, col=1)

        fig.update_layout(height=600, width=1600, title_text="Weights Composition",showlegend=True)
        fig.update_xaxes(rangeslider_visible=True)
        return fig


def generate_simple_timeseries_plot(data = None, title = ''):
    fig = make_subplots(rows=1, cols=1)
    for me_constituent in data.columns:
        trace_sig = go.Scatter(
            x=data.index,
            y=data[me_constituent],
            name=me_constituent,
            opacity=0.8)
        fig.append_trace(trace_sig, row=1, col=1)
    fig.update_layout(height=600, width=1600, title_text=title, showlegend=True)
    fig.update_xaxes(rangeslider_visible=True)
    return fig



__author__ = "hugo.inzirillo"

import six

from napoleontoolbox.napoleon_config_tools.object.config import Config
from npxlogger import log


class _Profile(object):

    def __init__(self, value: str):
        if isinstance(value, six.string_types):
            self.__value = value
        else:
            raise log.exception("Profile value should be string typed")

    def __call__(self, o: type):
        try:
            if isinstance(o, object):
                o.config = Config()
                o.config.package = str(__package__).split(".")[0]
                o.config.profile = self.__value

                return o
            else:
                raise TypeError("Not object typed")
        except Exception as e:
            raise e


Profile = _Profile



__author__ = "hugo.inzirillo"

from napoleontoolbox.napoleon_config_tools.parser import target, config, getter

__all__ = ['target', 'config', 'getter']






if __name__ == '__main__':
    end = True


__author__ = "hugo.inzirillo"

from npxlogger.logger_manager import log

__all__ = ['log']
__version__ = "0.0.1"

########################## EXEMPLE ######################


# from npxlogger import log
# log.info()
# log.warning()
# log.error()
# log.warn()



#!/usr/bin/env python
# coding: utf-8

from abc import ABC, abstractmethod

import pandas as pd
import numpy as np

from napoleontoolbox.neural_net import roll_multi_layer_lstm
from napoleontoolbox.file_saver import dropbox_file_saver
from napoleontoolbox.neural_net import roll_multi_layer_perceptron
from napoleontoolbox.boosted_trees import roll_lightgbm
from napoleontoolbox.features import features_type
from napoleontoolbox.utility import utility

import torch
import torch.nn as nn

class AbstractRunner(ABC):
    def __init__(self, starting_date = None, running_date = None, drop_token=None, dropbox_backup = True, supervision_npy_file_suffix='_supervision.npy', macro_supervision_npy_file_suffix='_macro_supervision.npy', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy', returns_pkl_file_name='returns.pkl', shortcut_features_names_saving_suffix = '_shortcut_features_names.npy', shortcut_features_saving_suffix='_shortcut_features.npy', local_root_directory='../data/', user = 'napoleon', n_start = 252, maximum_precision=True, shortcut_features=False):
        super().__init__()
        self.supervision_npy_file_suffix=supervision_npy_file_suffix
        self.macro_supervision_npy_file_suffix=macro_supervision_npy_file_suffix
        self.features_saving_suffix=features_saving_suffix
        self.features_names_saving_suffix=features_names_saving_suffix
        self.returns_pkl_file_name=returns_pkl_file_name
        self.local_root_directory=local_root_directory
        self.shortcut_features_saving_suffix = shortcut_features_saving_suffix
        self.shortcut_features_names_saving_suffix = shortcut_features_names_saving_suffix
        self.shortcut_features = shortcut_features
        self.user=user
        self.n_start = n_start
        self.dropbox_backup = dropbox_backup
        self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token=drop_token,dropbox_backup=dropbox_backup)
        self.starting_date = starting_date
        self.running_date = running_date
        self.maximum_precision = maximum_precision

    @abstractmethod
    def runTrial(self,saver, seed, sup, layers, epochs, n_past_features, n, s, whole_history, advance_feature, advance_signal,
                 normalize, activation_string, convolution, lr_start, lr_type, low_bound, up_bound):
        pass

class SimpleExplainer(AbstractRunner):
    def runTrial(self, saver, seed, sup, layers, epochs, n_past_features, n, s, feature_type, activation_type,
                 convolution, lr_start, lr_type, low_bound, up_bound):

        if (
                feature_type is features_type.FeaturesType.HISTORY or feature_type is features_type.FeaturesType.HISTORY_ADVANCED) and n_past_features is None:
            return
        if (
                feature_type is features_type.FeaturesType.STANDARD or feature_type is features_type.FeaturesType.STANDARD_ADVANCED) and n_past_features is not None:
            return

        meArg = (
            seed, sup, layers, epochs, n_past_features, n, s, feature_type,
            activation_type, convolution, lr_start, lr_type, low_bound, up_bound)

        meArgList = list(meArg)
        meArgList = [str(it) for it in meArgList]
        savingKey = ''.join(meArgList)
        savingKey = savingKey.replace('[', '')
        savingKey = savingKey.replace(']', '')
        savingKey = savingKey.replace(',', '')
        savingKey = savingKey.replace(' ', '')
        savingKey = savingKey.replace('.', '')
        # savingKey = savingKey.replace('_','')
        savingKey = 'T_' + savingKey

        print('Launching computation with parameters : ' + savingKey)

        supervisors = {}
        supervisors['f_minVar'] = 0
        supervisors['f_maxMean'] = 1
        supervisors['f_sharpe'] = 2
        supervisors['f_MeanVar'] = 3
        supervisors['f_calmar'] = 4
        supervisors['f_drawdown'] = 5

        df = self.dbx.local_overwrite_and_load_pickle(folder='', subfolder='',
                                                      returns_pkl_file_name=self.returns_pkl_file_name,
                                                      local_root_directory=self.local_root_directory)

        df['Date'] = pd.to_datetime(df['Date'])
        df = df.set_index('Date')
        df = df[df.index>=self.starting_date]
        df = df[df.index<=self.running_date]

        dates = df.index
        df = df.fillna(method='ffill')
        print('size return ' + str(df.shape))

        result = self.dbx.local_supervision_npy_overwrite_and_load_array(starting_date=self.starting_date, ending_date=self.running_date, rebal=s, lower_bound=low_bound,
                                                                         upper_bound=up_bound,
                                                                         local_root_directory=self.local_root_directory,
                                                                         user=self.user,
                                                                         supervision_npy_file_suffix=self.supervision_npy_file_suffix)

        np.random.seed(seed)
        torch.manual_seed(seed)
        # Set data
        whole_history = (
                    feature_type is features_type.FeaturesType.HISTORY or feature_type is features_type.FeaturesType.HISTORY_ADVANCED)

        features, features_names, shortcut_features, shortcut_features_names = self.dbx.local_features_npy_ovverwrite_and_load_array(starting_date= self.starting_date, ending_date = self.running_date,feature_type=feature_type, n_past_features=n_past_features,
                                                    local_root_directory=self.local_root_directory, user=self.user,
                                                    features_saving_suffix=self.features_saving_suffix,
                                                    features_names_saving_suffix=self.features_names_saving_suffix,
                                                    shortcut_features_saving_suffix=self.shortcut_features_saving_suffix,
                                                    shortcut_features_names_saving_suffix=self.shortcut_features_names_saving_suffix)


        assert shortcut_features.shape[0] == features.shape[0]

        # the save files necessarily match the beginning date
        shortcut_features = shortcut_features[:len(df)]
        features = features[:len(df)]
        result = result[:len(df)]

        X = features[self.n_start:]
        y = result[self.n_start:, :, supervisors[sup]]
        df = df.iloc[self.n_start:]
        dates = dates[self.n_start:]

        print('predictors')
        print(X.shape)
        print('utility')
        print(y.shape)
        print('prices')
        print(df.shape)
        print('dates')
        print(len(dates))

        # convolution 0 : perceptron
        # convolution 1 : LSTM
        # convolution 2 : xgboost

        if whole_history:
            if convolution == 2:
                print('no whole time history with ensembling method')
                return
            if convolution == 0:
                print('no whole time history with multi layers perceptron')
                # uncomment if you want multi layers perceptron with full historical backtest
                # print('flattening predictor time series for perceptron')
                # _X = np.empty((X.shape[0], X.shape[1] * X.shape[2]), dtype=np.float32)
                # for l in range(X.shape[0]):
                #     temp = np.transpose(X[l, :, :])
                #     _X[l, :] = temp.flatten()
                # X = _X

        if not whole_history:
            if convolution == 1:
                print('only whole time history with lstm')
                # print('adding one virtual time stamp')
                # X = X[..., np.newaxis, :]
                return

        print('number of nan/infinity features')
        print(np.isnan(X).sum(axis=0).sum())
        print(np.isinf(X).sum(axis=0).sum())
        print('number of nan/infinity output')
        print(np.isnan(y).sum(axis=0).sum())
        print(np.isinf(y).sum(axis=0).sum())

        print(np.count_nonzero(~np.isnan(X)))

        if self.maximum_precision:
            X = np.float64(X)
            y = np.float64(y)
        else :
            X = np.float32(X)
            y = np.float32(y)


        neural_net_precision = None
        if X.dtype == np.float64:
            neural_net_precision = torch.float64
        if X.dtype == np.float32:
            neural_net_precision = torch.float32

        activation_function = utility.convertActivationType(activation_type)

        if convolution == 1:
            # the number of figures
            input_size = X.shape[2]
            hidden_size = int(layers[-1]/2)
            num_layers = 1
            num_classes = y.shape[1]
            tm = roll_multi_layer_lstm.RollMultiLayerLSTM(
                X=X,
                y=y,
                num_classes=num_classes,
                input_size=input_size,
                hidden_size=hidden_size,
                num_layers=num_layers,
                # nn.Softmax/nn.Softmin can be good activations for this problem
                x_type=neural_net_precision,
                y_type=neural_net_precision
                # activation_kwargs={'dim':1} # Parameter needed for nn.Softmax/nn.Softmin
            )
            tm.set_optimizer(nn.MSELoss, torch.optim.Adam, lr_type, lr=lr_start, betas=(0.9, 0.999), amsgrad=True)
            tm = tm.set_roll_period(n, s, repass_steps=epochs)
        elif convolution == 0:
            tm = roll_multi_layer_perceptron.RollMultiLayerPerceptron(
                X=X,
                y=y,
                layers=layers,
                activation=activation_function,  # nn.Softmax/nn.Softmin can be good activations for this problem
                x_type=neural_net_precision,
                y_type=neural_net_precision,
                # activation_kwargs={'dim':1} # Parameter needed for nn.Softmax/nn.Softmin
            )
            tm.set_optimizer(nn.MSELoss, torch.optim.Adam, lr_type, lr=lr_start, betas=(0.9, 0.999), amsgrad=True)
            tm = tm.set_roll_period(n, s, repass_steps=epochs)
        elif convolution == 2:
            tm = roll_lightgbm.RollLightGbm(
                X=X,
                y=y
            )
            tm = tm.set_roll_period(n, s)

        features_df, predictions_df, returns_df = tm.unroll_features(dates, features_names)
        return features_df, predictions_df, returns_df

    def ratio_computer(self,features_df=None, features_segmentation = {}, tol = 1e-6):
        features_names = list(features_df.columns)
        extended_features_segmentation = {}
        for key, values in features_segmentation.items():
            selected_columns = []
            for feature_value in values:
                selected_columns.extend([col for col in features_names if feature_value in col])
                extended_features_segmentation[key] = selected_columns

        # aggregating features importance
        features_importance = features_df.sum(axis=0)
        features_importance = pd.DataFrame(
            {'col': features_importance.index, 'features_importance': features_importance.values})
        features_importance.sort_values('features_importance', inplace=True, ascending=False)

        all_group_features_importance = None
        for key, features_group in extended_features_segmentation.items():
            print('Features group ' + key)
            print('Number of  features for this group ' + str(len(features_group)))
            features_bool_indexing = [True if col in features_group else False for col in features_importance.col]
            group_features_importance = features_importance[features_bool_indexing]
            print('merging in a common dataframe')
            group_features_importance['group'] = key
            if all_group_features_importance is None:
                all_group_features_importance = group_features_importance
            else:
                all_group_features_importance = all_group_features_importance.append(group_features_importance,
                                                                                     ignore_index=True)

        features_importance_summarized = all_group_features_importance.groupby('group')['features_importance'].sum()
        features_importance_summarized = features_importance_summarized / features_importance_summarized.sum()
        if 'Dates' in features_df:
            dates = features_df.Dates
        else:
            dates = pd.to_datetime(features_df.index)

        part_over_time_df = None

        for key, features_values in extended_features_segmentation.items():
            grouped_features_df = features_df[features_values]
            if part_over_time_df is None:
                temp = (grouped_features_df.abs()+tol).sum(axis=1)
                temp.columns = [key]
                part_over_time_df = temp
            else:
                temp = (grouped_features_df.abs()+tol).sum(axis=1)
                temp.columns = [key]
                new_columns = list(part_over_time_df.columns)+[key]
                part_over_time_df = pd.concat([part_over_time_df, temp],axis=1)
                part_over_time_df.columns = new_columns



        part_over_time_df = part_over_time_df.div(part_over_time_df.sum(axis=1), axis=0)

        part_over_time_df.index = dates
        return features_importance_summarized, part_over_time_df











__author__ = "hugo.inzirillo"

from abc import abstractmethod, ABCMeta
from typing import Union

from napoleontoolbox.authentication import ClientCredentials, PasswordCredentials, Token


class AbstractHeadersTemplate(metaclass=ABCMeta):

    @property
    @abstractmethod
    def value(self):
        raise NotImplemented

    @staticmethod
    def build(object: Union[PasswordCredentials, ClientCredentials, Token]):
        raise NotImplemented


class AbstractHeaders(AbstractHeadersTemplate):

    @property
    @abstractmethod
    def value(self):
        raise NotImplemented

    @staticmethod
    def build(object: Union[PasswordCredentials, ClientCredentials, Token]):
        raise NotImplemented


class AbstractHeadersScheme(AbstractHeadersTemplate):

    @property
    @abstractmethod
    def value(self):
        raise NotImplemented

    @staticmethod
    def build(object: Union[PasswordCredentials, ClientCredentials, Token]):
        raise NotImplemented


class HeadersScheme(AbstractHeadersTemplate):

    def __init__(self):
        self.__value = None

    def value(self):
        pass

    @staticmethod
    def build(object: Union[PasswordCredentials, ClientCredentials, Token]):
        pass


class TemplateHeaders(AbstractHeaders):

    def value(self): ...

    @staticmethod
    def build(object: Union[PasswordCredentials, ClientCredentials, Token]): ...


class Headers(TemplateHeaders):
    def __init__(self):
        self.__content = None
        self.__value = dict()

    @property
    def value(self) -> dict:
        return self.__value

    def __dict__(self):
        return self.__value

    @staticmethod
    def build(o: Union[PasswordCredentials, ClientCredentials, Token]):
        return HeaderGenerator().build(o).value


class AuthenticationHeader(TemplateHeaders):
    pass


class HeaderGenerator(AbstractHeaders):
    def __init__(self):
        self.__header = None

    @property
    def value(self):
        return self.__header

    def build(self, object: Union[PasswordCredentials, ClientCredentials, Token]):
        if isinstance(self.__header.content, ClientCredentials):
            return None
        elif isinstance(self.__header.content, PasswordCredentials):
            return None
        elif isinstance(self.__header.content, Token):
            return None
        return self



__author__ = "hugo.inzirillo"

import os

import yaml

from napoleontoolbox import __path__
from napoleontoolbox.napoleon_config_tools.loader.abstractreader import AbstractYamlReader
from napoleontoolbox.napoleon_config_tools.object.config import Config
from npxlogger import log


class YamlReader(AbstractYamlReader):
    def __init__(self):
        pass

    def _read(self, _path: str) -> dict:
        """

        Parameters
        ----------
        _path : path of file to read .yml

        Returns
        -------

        """
        with open(_path, 'r') as stream:
            try:
                _config = yaml.safe_load(stream)
                log.info("reading current config : {conf}".format(conf=_config))
                return _config
            except yaml.YAMLError as exc:
                log.error(exc)


class NapoleonConfigReader(YamlReader):

    @property
    def __config_file_path(self):
        return os.path.join(__path__[0], Config.FILE)

    def _read(self, _path: str = None) -> dict:
        """

        Parameters
        ----------
        _path : path of file to read .yml

        Returns
        -------

        """

        if _path is not None:
            path = _path
        else:
            path = self.__config_file_path

        return super(NapoleonConfigReader, self)._read(path)

    def read(self, _path: str = None) -> dict:
        """
        This method read the config.yml file
        Returns
        -------

        """

        return self._read(_path)
